#ifndef RAST_I_H_
#define RAST_I_H_

#include "raster.h"

//
// internal usage rasterizer header
//



//
// determines size of statically allocated buffers
//
const int max_x_res = 640;
const int max_y_res = 480;

namespace raster
{
 extern int span_extents[max_y_res][2];
 extern int y_min, y_max;

 //
 // edge tracing
 //

 inline void reset_spans()
 {
  y_max = 0;
  y_min = max_y_res;
 }
 typedef void  __fastcall  trace_edge_func_t(float fx0, float fy0, float fx1,float fy1);

 extern trace_edge_func_t * trace_edge_func;
 

 extern int bytes_per_pixel;
 
//
// pointers to each row of current frame buffer
// 
 extern u16  * frame_buf_starts[max_y_res];

 // used for 16 bit color
 extern u32 pixel_blend_mask;

 //
 // poly linear basises
 //
 extern float plb_sh[3];
 extern float plb_u[3];
 extern float plb_v[3];
 extern float plb_z[3];

//
// used in trace_edge_clipped
// 
 extern int   clip_max_x_fx16;
 extern float clip_max_x_flt;
 extern int x_res, y_res;   


 //
 // sets perspective draw func based on 
 // block_dimf and bytes_per_pixel
 extern void refresh_persp_draw_funcs();
 extern int block_bits;


 //
 // scan-line fill functions
 //
 typedef void __fastcall fill_text_func(const texture *  );
 typedef void __fastcall fill_solid_func(const void * clut);
 typedef void __fastcall fill_flat_solid_func(const void * clut,int ishade, int is_translucent );


 extern fill_text_func ** fill_alt;
 extern fill_text_func ** fill_aft;
 extern fill_solid_func * fill_als;
 extern fill_flat_solid_func * fill_afs;



};
 




//
//  note: only use this function if you are sure that poly is
//  not clipped
//
void __fastcall trace_edge_not_clipped(float fx0, float fy0, float fx1,float fy1);
 
//
// does 2d clippping to current screen rect (0,0..raster::x_res,raster::y_res)
//
void __fastcall trace_edge_clipped(float fx0, float fy0, float fx1,float fy1); 



// 
//  setting floating point unit
//
const short math_cw_double =  0x27f;
const short math_cw_single =  0x7f;

inline void set_single_pres()
{
 __asm 
 {
  fldcw math_cw_single;
 }
}
inline void set_double_pres()
{
 __asm 
 {
  fldcw math_cw_double;
 }
}

#ifdef FP_SINGLE_PRES

#define INIT_FP     set_double_pres();
#define RESTORE_FP  set_single_pres();
#define RESTORE_FP_FROM_SINGLE

#else

#define INIT_FP
#define RESTORE_FP
#define RESTORE_FP_FROM_SINGLE  set_double_pres();

#endif


// to avoid under/overflow during interpolation
const float linear_shade_eps   = 1.0f/96.0f;
const float linear_shade_scale = 1.0 - 2.0*linear_shade_eps;


//
//  glorified 2x2 matrix multiply
//  MSVC handles this macro version better than an inline function
#define LB_TO_SCREEN(D,S,X0,Y0)                   \
 {                                                \
  float ds0 =  (S)[1] - (S)[0] ;                  \
  float ds1 =  (S)[2] - (S)[0] ;                  \
  D[1] = xf_mat00*ds0 + xf_mat10*ds1;             \
  D[2] = xf_mat01*ds0 + xf_mat11*ds1;             \
  D[0] = (S)[0] - (D[1]) * (X0) - (D[2])*(Y0);    \
 }

//
//  same as above, but for linear shades, which must be 
//  scaled to prevent under/overflow
//
#define LB_TO_SCREEN_SH(D,S,X0,Y0)                   \
 {                                                \
  float ds0 =  (S)[1] - (S)[0];                  \
  float ds1 =  (S)[2] - (S)[0];                  \
  D[1] = (xf_mat00*ds0 + xf_mat10*ds1)*linear_shade_scale;             \
  D[2] = (xf_mat01*ds0 + xf_mat11*ds1)*linear_shade_scale;             \
  D[0] = (S)[0]*linear_shade_scale - (D[1]) * (X0) - (D[2])*(Y0) + linear_shade_eps;    \
 }


//
//  same as LB_TO_SCREEN but applies perspective adjust
//
#define LB_TO_SCREEN_P(D,S,X0,Y0)                   \
 {                                                  \
  float s0 =   (S)[0]*z_s[0];                          \
  float ds0 =  (S)[1]*z_s[1] - s0;                  \
  float ds1 =  (S)[2]*z_s[2] - s0;                  \
  D[1] = xf_mat00*ds0 + xf_mat10*ds1;             \
  D[2] = xf_mat01*ds0 + xf_mat11*ds1;             \
  D[0] = s0 - (D[1]) * (X0) - (D[2])*(Y0);    \
 }


//
// to avoid numerical unstability in division by determinant
//
const float min_tri_area = (0.125);


#endif